--[[

  Hacks used:
  
    Shifted graphic + selection/collision box down
    to prevent graphic from rendering above player
    as render_layer is not supported for container.

    Entity name/description is dynamic but item
    name/desc is used from actual [entity-name] locale entry.
    
  ]]

local stockpile = table.deepcopy(data.raw.container['steel-chest'])

local modpath = '__eradicators-stockpile__'

local function add_stockpile(width,slot_count,name,icon_overlay,size_postfix)
  
  local scale = 1 /( 640 / (width*32)) --unscaled picture width is 20.0
 
  local connector = circuit_connector_definitions.create(universal_connector_template,{{
    variation = 0,
    -- main_offset = util.by_pixel(3, 5.5),
    -- shadow_offset = util.by_pixel(7.5, 7.5),
    main_offset   = { 1*(width/2 - 0.1) - scale*0.5, 1*(width/2 - 0.1) - scale*1.5},
    shadow_offset = { 1*(width/2 - 0.1) - scale*0.5, 1*(width/2 - 0.1) - scale*1.5},
    show_shadow = true
    }})

  local entity = {
    type = "container", name = name,
    icon = modpath .. '/icon.png', icon_size = 64,
    flags = {"placeable-neutral", "player-creation","hide-alt-info"},
    minable = {mining_time = 0.75, result = name},
    max_health = width * 100,
    order = "a[items]-k[stockpile]-"..string.format('%.5d',width),
    placeable_by = {item = name, count = 1},
    
    corpse = "small-remnants",
    open_sound  = { filename = "__base__/sound/metallic-chest-open.ogg" , volume = 0.65 },
    close_sound = { filename = "__base__/sound/metallic-chest-close.ogg", volume = 0.7  },
    
    resistances = table.deepcopy(data.raw.container['steel-chest'].resistances),
    fast_replaceable_group = "container",
    
    collision_box = {
      {-1*(width/2 - 0.24), -1*(width/2 - 0.24)},
      { 1*(width/2 - 0.24),  1*(width/2 - 0.24)}
      },
    selection_box = {
      {-1*(width/2 + 0.24), -1*(width/2 + 0.24)},
      { 1*(width/2 + 0.24),  1*(width/2 + 0.24)}
      },
      
    inventory_size = slot_count,
    
    circuit_wire_connection_point = connector.points,
    circuit_connector_sprites = connector.sprites,
    circuit_wire_max_distance = default_circuit_wire_max_distance,
    
    picture = { layers = {
      { priority = "extra-high",
        filename = modpath .. '/640_wall.png', width = 640, height = 652,
        -- shift = util.by_pixel(32*0.5*width, 0),
        shift = {0.3*scale, 0.1*scale},
        scale = scale,
        },
      { priority = "extra-high",
        filename = modpath .. '/640_shadow.png', width = 640, height = 652,
        -- shift = util.by_pixel(32*0.5*width, 0),
        shift = {0.3*scale, 0.1*scale},
        scale = scale,
        draw_as_shadow = true,
        }
      } },
    }
    
  local function clone(postfix,mode,max_logistic_slots,sorting,tint)
    local this = util.table.deepcopy(entity)
    this.name = this.name .. postfix
    this.logistic_mode = mode
    this.max_logistic_slots = max_logistic_slots
    if mode then this.type = 'logistic-container' end
    if tint then this.picture.layers[1].tint = tint end
    local lsize = {'er.stockpile-size'..size_postfix}
    local ltype = mode and {'er.stockpile-type-logistic'} or ((sorting == 'linear') and {'er.stockpile-type-linear'} or {'er.stockpile-type-chaotic'})
    local log_mode = mode and {'er.stockpile-logistic-postfix',{'er.logistic-mode-'..mode}} or ''
    this.localised_name = {'er.stockpile-name',lsize,ltype,log_mode}
    this.localised_description = {'er.stockpile-description'}
    return this
    end
    
  local chaotic = clone(''                 , nil              ,nil ,'chaotic')
  local linear  = clone('-linear'          , nil              ,nil ,'linear' )
  local storage = clone('-storage'         ,'storage'         ,1   ,'chaotic',{r=192,g=146,b= 68,a=255})
  local passive = clone('-passive-provider','passive-provider',nil ,'chaotic',{r=139,g= 50,b= 34,a=255})
  -- print(serpent.block(storage)) error()
    
  local item = {
    type = "item",
    name = name,
    -- icon = modpath .. '/icon.png', icon_size = 64,
    icons = {
      {icon_size = 64, icon = modpath .. '/icon.png'},
      {icon_size = 64, icon = modpath .. icon_overlay},
      },
    subgroup = "storage",
    order = "a[items]-k[stockpile]-"..string.format('%.5d',width),
    place_result = name,
    stack_size = 50
    }
    
  local recipe = {
    type = "recipe",
    name = name,
    enabled = false,
    ingredients = {{"stone-wall", 4 + width * 4}},
    result = name,
    }
    
  data:extend{chaotic,linear,storage,passive,item,recipe}
 
  --always enabled if stone-walls tech was modified
  local tech = data.raw.technology['stone-walls']
  if tech and tech.effects and tech.effects[1] and tech.effects[1].recipe == 'stone-wall' then
    table.insert(tech.effects,{type='unlock-recipe',recipe=name})    
  else
    recipe.enabled = true
    end

  end

-- add_stockpile( 3, 3^2,'er:stockpile-small' , '/icon_overlay_circle-one.png'  )
add_stockpile( 5, 5^2,'er:stockpile-medium', '/icon_overlay_circle-one.png'   ,'-medium')
add_stockpile(12,10^2,'er:stockpile-large' , '/icon_overlay_circle-two.png'   ,'-large' )
add_stockpile(20,18^2,'er:stockpile-huge'  , '/icon_overlay_circle-three.png' ,'-huge'  )

--link "rotate"
local function add_hotkey (name,key,link)
  if (data.raw['custom-input']) and (not data.raw['custom-input'][name]) then
    data:extend{{
      type                = 'custom-input',
      name                = name or error('Missing name for hotkey.'),
      linked_game_control = link or nil,
      key_sequence        = key or '',
      }}
    end
  end 
    
add_hotkey('er:linked-control:rotate'        ,nil,'rotate'        )
add_hotkey('er:linked-control:reverse-rotate',nil,'reverse-rotate')
